/*
 * Copyright (C) 2006 Zack Rusin <zack@kde.org>
 * Copyright (C) 2006 Apple Computer, Inc.  All rights reserved.
 * Copyright (C) 2008 Nokia Corporation and/or its subsidiary(-ies)
 * Copyright (C) 2008 Collabora Ltd. All rights reserved.
 * Copyright (C) 2008 Holger Hans Peter Freyther
 * Copyright (C) 2008 Kenneth Rohde Christiansen
 * Copyright (C) 2009-2010 ProFUSION embedded systems
 * Copyright (C) 2009-2010 Samsung Electronics
 * Copyright (c) 2012 NINTENDO.
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE COMPUTER, INC. ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL APPLE COMPUTER, INC. OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "config.h"
#include "FrameLoaderClientWiiU.h"

#include "DocumentLoader.h"
#include "FormState.h"
#include "FrameLoader.h"
#include "FrameNetworkingContextWiiU.h"
#include "FrameTree.h"
#include "FrameView.h"
#include "Page.h"
#include "HTMLFormElement.h"
#include "Language.h"
#include "MIMETypeRegistry.h"
#include "NotImplemented.h"
#include "PluginDatabase.h"
#include "ProgressTracker.h"
#include "RenderPart.h"
#include "ResourceHandleManager.h"
#include "ResourceRequest.h"
#include "ViewportArguments.h"
#include "Credential.h"
#include "CredentialStorage.h"
#include <wtf/text/CString.h>
#include <wtf/text/StringBuilder.h>

#if PLATFORM(UNIX)
#include <sys/utsname.h>
#endif

#include <PlatformOS.h>
#include <PlatformError.h>
#include <PlatformInput.h>
using namespace LibPlatform;

//#define notImplemented(x) OS::Report("not implemented: %s %d\n", __FUNCTION__, __LINE__)

using namespace WebCore;

namespace WebCore {

FrameLoaderClientWiiU::FrameLoaderClientWiiU(WebView *view)
    : m_view(view)
    , m_frame(0)
    , m_userAgent("")
    , m_customUserAgent("")
    , m_pluginView(0)
    , m_hasSentResponseToPlugin(false)
    , m_initLayoutCompleted(false)
{
}

static String agentPlatform()
{
    notImplemented();
    return "Unknown";
}

static String agentOS()
{
    notImplemented();
    return "Unknown";
}

static String composeUserAgent()
{
    // This is a liberal interpretation of http://www.mozilla.org/build/revised-user-agent-strings.html
    // See also http://developer.apple.com/internet/safari/faq.html#anchor2

    StringBuilder ua;

#if 0
    // Product
    ua.appendLiteral("Mozilla/5.0");

    // Comment
    ua.appendLiteral(" (");
    ua.append(agentPlatform()); // Platform
    ua.appendLiteral("; U; "); // Security
    ua.append(agentOS()); // OS-or-CPU
    ua.appendLiteral("; ");
    ua.append(defaultLanguage()); // Localization information
    ua.appendLiteral(") ");

    // WebKit Product
    // FIXME: The WebKit version is hardcoded
    const String webKitVersion = "525.1+";
    ua.appendLiteral("AppleWebKit/");
    ua.append(webKitVersion);
    ua.appendLiteral(" (KHTML, like Gecko, ");
    // We mention Safari since many broken sites check for it (OmniWeb does this too)
    // We re-use the WebKit version, though it doesn't seem to matter much in practice
    ua.appendLiteral("Safari/");
    ua.append(webKitVersion);
    ua.appendLiteral(") ");

    // Vendor Product
    // ua += g_get_prgname();
#else
	ua.appendLiteral("Mozilla/5.0 (Nintendo WiiU) AppleWebKit/534.53 (KHTML, like Gecko) NWF/1.0");
#endif

    return ua.toString();
}

PassRefPtr<Frame> FrameLoaderClientWiiU::createFrame(const KURL& url, const String& name, HTMLFrameOwnerElement* ownerElement, const String& referrer,
                                       bool /*allowsScrolling*/, int /*marginWidth*/, int /*marginHeight*/)
{
    FrameLoaderClientWiiU *loaderClient = new FrameLoaderClientWiiU(m_view);
    RefPtr<Frame> childFrame = Frame::create(m_frame->page(), ownerElement, loaderClient);
    loaderClient->setWebFrame(childFrame.get());

    m_frame->tree()->appendChild(childFrame);
    childFrame->tree()->setName(name);
    childFrame->init();

    // The creation of the frame may have run arbitrary JavaScript that removed it from the page already.
    if (!childFrame->page())
        return 0;

    childFrame->loader()->loadURLIntoChildFrame(url, referrer, childFrame.get());

    // The frame's onload handler may have removed it from the document.
    if (!childFrame->tree()->parent())
        return 0;

    return childFrame.release();
}

void FrameLoaderClientWiiU::didTransferChildFrameToNewDocument(Page*)
{
}

void FrameLoaderClientWiiU::didSaveToPageCache()
{
}

void FrameLoaderClientWiiU::didRestoreFromPageCache()
{
}

void FrameLoaderClientWiiU::transferLoadingResourceFromPage(ResourceLoader*, const ResourceRequest&, Page* oldPage)
{
}


void FrameLoaderClientWiiU::setCustomUserAgent(const String &agent)
{
    m_customUserAgent = WebCore::CleanUpHTTPHeaderValue(agent);
}

const String& FrameLoaderClientWiiU::customUserAgent() const
{
    return m_customUserAgent;
}

String FrameLoaderClientWiiU::userAgent(const KURL&)
{
	if (!m_customUserAgent.isEmpty())
        return m_customUserAgent;

    if (m_userAgent.isEmpty())
        m_userAgent = composeUserAgent();
    return m_userAgent;
}

void FrameLoaderClientWiiU::setCustomHeader(const String &header)
{
	//This is just a temp fix! m_impl is getting screwed up in the string copy somehow somewhere,
	//so just create a whole new string.
	LibPlatform::OS::Report("++Setting custom header: %d bytes. (this = 0x%X)\n", header.length(), this);
	m_customHeader.push_back(header.ascii().buffer()->data());
}

std::vector<String> FrameLoaderClientWiiU::customHeaders()
{
	return m_customHeader;
}

void FrameLoaderClientWiiU::callPolicyFunction(FramePolicyFunction function, PolicyAction action)
{
    (m_frame->loader()->policyChecker()->*function)(action);
}

WTF::PassRefPtr<DocumentLoader> FrameLoaderClientWiiU::createDocumentLoader(const ResourceRequest& request, const SubstituteData& substituteData)
{
    RefPtr<DocumentLoader> loader = DocumentLoader::create(request, substituteData);
    if (substituteData.isValid())
        loader->setDeferMainResourceDataLoad(false);
    return loader.release();
}

void FrameLoaderClientWiiU::dispatchWillSubmitForm(FramePolicyFunction function, PassRefPtr<FormState>)
{
    // FIXME: This is surely too simple
    ASSERT(function);
    callPolicyFunction(function, PolicyUse);
}

void FrameLoaderClientWiiU::committedLoad(DocumentLoader* loader, const char* data, int length)
{
    if (!m_pluginView)
        loader->commitData(data, length);

	if( m_frame->document()->isMediaDocument() )
	{
		loader->cancelMainResourceLoad( loader->mainDocumentError() );
	}
#if 0
    // We re-check here as the plugin can have been created
    if (m_pluginView) {
        if (!m_hasSentResponseToPlugin) {
            m_pluginView->didReceiveResponse(loader->response());
            m_hasSentResponseToPlugin = true;
        }
        m_pluginView->didReceiveData(data, length);
    }
#endif
}

void FrameLoaderClientWiiU::dispatchDidReplaceStateWithinPage()
{
    notImplemented();
}

void FrameLoaderClientWiiU::dispatchDidRemoveBackForwardItem(WebCore::HistoryItem*) const
{
    notImplemented();
}

bool FrameLoaderClientWiiU::shouldStopLoadingForHistoryItem(HistoryItem* item) const
{
    return true;
}

void FrameLoaderClientWiiU::dispatchDidPushStateWithinPage()
{
    notImplemented();
}

void FrameLoaderClientWiiU::dispatchDidPopStateWithinPage()
{
    notImplemented();
}

void FrameLoaderClientWiiU::dispatchDidChangeBackForwardIndex() const
{
    notImplemented();
}

void FrameLoaderClientWiiU::dispatchDidAddBackForwardItem(WebCore::HistoryItem*) const
{
    notImplemented();
}

void FrameLoaderClientWiiU::dispatchDidClearWindowObjectInWorld(DOMWrapperWorld*)
{
	//APW - Since this callback now works I moved the extension loading code to here instead of WebView::progress
	//This function is important because it is the first WebView related callback after the JSGlobalObject has been initted
	//I got this info from http://rvr.typepad.com/wind/2011/10/webkit-extending-javascript-2.html
	m_view->loadExtensionRPLList(m_frame);
	
	//This event tells the user that the window has had all of the config.xml extensions added to it
	m_view->SetEventState(WEBVIEW_EVENT_INIT);
}

void FrameLoaderClientWiiU::dispatchDidReceiveAuthenticationChallenge(DocumentLoader* loader,  unsigned long  identifier, const AuthenticationChallenge& auth)
{
		char *user = NULL;
		char *password = NULL;
	    bool ret = false;

	    WebCore::CredentialPersistence persistence = CredentialPersistenceForSession;

	    ret = m_view->authenticate(loader->url().string().utf8().data(), &user, &password);

	    if (ret)
	    {
	        Credential credential(user, password, persistence);
	        CredentialStorage::set(credential, auth.protectionSpace(), loader->url());
	    }

	    if (user != NULL)
	    {
	    	free(user);
	    }

	    if (password != NULL)
	    {
	    	free(password);
	    }
}

void FrameLoaderClientWiiU::dispatchDidCancelAuthenticationChallenge(DocumentLoader*, unsigned long  identifier, const AuthenticationChallenge&)
{
    notImplemented();
}

void FrameLoaderClientWiiU::dispatchWillSendRequest(DocumentLoader* loader, unsigned long identifier, ResourceRequest& coreRequest, const ResourceResponse& coreResponse)
{
    notImplemented();
}

bool FrameLoaderClientWiiU::shouldUseCredentialStorage(DocumentLoader*, unsigned long)
{
    notImplemented();
    return false;
}

void FrameLoaderClientWiiU::assignIdentifierToInitialRequest(unsigned long identifier, DocumentLoader*, const ResourceRequest& coreRequest)
{
    notImplemented();
}

void FrameLoaderClientWiiU::postProgressStartedNotification()
{
    postProgressEstimateChangedNotification();
	m_view->status(WEBVIEW_LOAD_STARTED, m_frame->document()->url().string().utf8().data());
}

void FrameLoaderClientWiiU::postProgressEstimateChangedNotification()
{
	m_view->progress(m_frame->page()->progress()->estimatedProgress()*100);
}

void FrameLoaderClientWiiU::postProgressFinishedNotification()
{
	m_view->status(WEBVIEW_LOAD_COMPLETE, m_frame->document()->url().string().utf8().data());

	//This event tells the user that the window has completely finished loading
	m_view->SetEventState(WEBVIEW_EVENT_LOAD, true);
}

void FrameLoaderClientWiiU::frameLoaderDestroyed()
{
	m_view = 0;
    m_frame = 0;

    delete this;
}

void FrameLoaderClientWiiU::dispatchDidReceiveResponse(DocumentLoader*, unsigned long, const ResourceResponse& response)
{
    m_response = response;
}

void FrameLoaderClientWiiU::dispatchDecidePolicyForResponse(FramePolicyFunction function, const ResourceResponse& response, const ResourceRequest&)
{
    // we need to call directly here (currently callPolicyFunction does that!)
    ASSERT(function);
    if (canShowMIMEType(response.mimeType()))
        callPolicyFunction(function, PolicyUse);
    else
        callPolicyFunction(function, PolicyDownload);
}

void FrameLoaderClientWiiU::dispatchDecidePolicyForNewWindowAction(FramePolicyFunction function, const NavigationAction& action, const ResourceRequest&, PassRefPtr<FormState>, const String&)
{
    ASSERT(function);
    ASSERT(m_frame);
    // if not acceptNavigationRequest - look at Qt -> PolicyIgnore;
    // FIXME: do proper check and only reset forms when on PolicyIgnore
    m_frame->loader()->resetMultipleFormSubmissionProtection();
    callPolicyFunction(function, PolicyUse);
}

void FrameLoaderClientWiiU::dispatchDecidePolicyForNavigationAction(FramePolicyFunction function, const NavigationAction& action, const ResourceRequest& resourceRequest, PassRefPtr<FormState>)
{
    ASSERT(function);
    ASSERT(m_frame);
    // if not acceptNavigationRequest - look at Qt -> PolicyIgnore;
    // FIXME: do proper check and only reset forms when on PolicyIgnore

    PolicyAction policy;
        if (action.type() == NavigationTypeFormSubmitted || action.type() == NavigationTypeFormResubmitted) {
            m_frame->loader()->resetMultipleFormSubmissionProtection();
        }
        policy = PolicyUse;
    callPolicyFunction(function, policy);
}

PassRefPtr<Widget> FrameLoaderClientWiiU::createPlugin(const IntSize& pluginSize, HTMLPlugInElement* element, const KURL& url, const Vector<String>& paramNames, const Vector<String>& paramValues, const String& mimeType, bool loadManually)
{
    RefPtr<PluginView> pluginView = PluginView::create(m_frame, pluginSize, (Element *)element, url, paramNames, paramValues, mimeType, loadManually);

    if (pluginView->status() == PluginStatusLoadedSuccessfully)
        return pluginView;

    dispatchDidFailToStartPlugin(pluginView.get());

    return pluginView;
}

void FrameLoaderClientWiiU::redirectDataToPlugin(Widget* pluginWidget)
{
    ASSERT(!m_pluginView);
    m_pluginView = static_cast<PluginView*>(pluginWidget);
    m_hasSentResponseToPlugin = false;
}

PassRefPtr<Widget> FrameLoaderClientWiiU::createJavaAppletWidget(const IntSize&, HTMLAppletElement*, const KURL& baseURL,
                                                  const Vector<String>& paramNames, const Vector<String>& paramValues)
{
    notImplemented();
    return 0;
}

ObjectContentType FrameLoaderClientWiiU::objectContentType(const KURL& url, const String& mimeType, bool shouldPreferPlugInsForImages)
{
    if (url.isEmpty() && mimeType.isEmpty())
        return ObjectContentNone;

    // We don't use MIMETypeRegistry::getMIMETypeForPath() because it returns "application/octet-stream" upon failure
    String type = mimeType;
    if (type.isEmpty())
        type = MIMETypeRegistry::getMIMETypeForExtension(url.path().substring(url.path().reverseFind('.') + 1));

    if (type.isEmpty())
        return ObjectContentFrame;

    if (MIMETypeRegistry::isSupportedImageMIMEType(type))
        return ObjectContentImage;

#if 0 // PluginDatabase is disabled until we have Plugin system done.
    if (PluginDatabase::installedPlugins()->isMIMETypeRegistered(mimeType))
        return ObjectContentNetscapePlugin;
#endif

    if (MIMETypeRegistry::isSupportedNonImageMIMEType(type))
        return ObjectContentFrame;

    if (url.protocol() == "about")
        return ObjectContentFrame;

    return ObjectContentNone;
}

String FrameLoaderClientWiiU::overrideMediaType() const
{
    notImplemented();
    return String();
}

void FrameLoaderClientWiiU::windowObjectCleared()
{
    notImplemented();
}

void FrameLoaderClientWiiU::documentElementAvailable()
{
    return;
}

void FrameLoaderClientWiiU::didPerformFirstNavigation() const
{
    notImplemented();
}

void FrameLoaderClientWiiU::registerForIconNotification(bool)
{
    notImplemented();
}

void FrameLoaderClientWiiU::setMainFrameDocumentReady(bool isReady)
{
    // this is only interesting once we provide an external API for the DOM
	//This event tells the user that the document can be accessed and edited using the window handle returned by the load function
	if(isReady)
		m_view->SetEventState(WEBVIEW_EVENT_READY);
}

bool FrameLoaderClientWiiU::hasWebView() const
{
    // notImplemented();
    return true;
}

bool FrameLoaderClientWiiU::hasFrameView() const
{
    notImplemented();
    return true;
}

void FrameLoaderClientWiiU::dispatchDidFinishLoad()
{
    m_loadError = ResourceError(); /* clears previous error */
}

void FrameLoaderClientWiiU::frameLoadCompleted()
{
    // Note: Can be called multiple times.
}

void FrameLoaderClientWiiU::saveViewStateToItem(HistoryItem* item)
{
    notImplemented();
}

void FrameLoaderClientWiiU::restoreViewState()
{
    ASSERT(m_frame);
    ASSERT(m_view);

}

void FrameLoaderClientWiiU::updateGlobalHistoryRedirectLinks()
{
}

bool FrameLoaderClientWiiU::shouldGoToHistoryItem(HistoryItem* item) const
{
    // FIXME: This is a very simple implementation. More sophisticated
    // implementation would delegate the decision to a PolicyDelegate.
    // See mac implementation for example.
    return item;
}

void FrameLoaderClientWiiU::didDisplayInsecureContent()
{
    notImplemented();
}

void FrameLoaderClientWiiU::didRunInsecureContent(SecurityOrigin*, const KURL&)
{
    notImplemented();
}

void FrameLoaderClientWiiU::makeRepresentation(DocumentLoader*)
{
    notImplemented();
}

void FrameLoaderClientWiiU::forceLayout()
{
    notImplemented();
}

void FrameLoaderClientWiiU::forceLayoutForNonHTML()
{
}

void FrameLoaderClientWiiU::setCopiesOnScroll()
{
    // apparently mac specific (Qt comment)
}

void FrameLoaderClientWiiU::detachedFromParent2()
{
}

void FrameLoaderClientWiiU::detachedFromParent3()
{
}

void FrameLoaderClientWiiU::loadedFromCachedPage()
{
    notImplemented();
}

void FrameLoaderClientWiiU::dispatchDidHandleOnloadEvents()
{
    notImplemented();
}

void FrameLoaderClientWiiU::dispatchDidReceiveServerRedirectForProvisionalLoad()
{
    notImplemented();
}

void FrameLoaderClientWiiU::dispatchDidCancelClientRedirect()
{
    notImplemented();
}

void FrameLoaderClientWiiU::dispatchWillPerformClientRedirect(const KURL&, double, double)
{
    notImplemented();
}

void FrameLoaderClientWiiU::dispatchDidChangeLocationWithinPage()
{
}

void FrameLoaderClientWiiU::dispatchWillClose()
{
    notImplemented();
}

void FrameLoaderClientWiiU::dispatchDidReceiveIcon()
{
}

void FrameLoaderClientWiiU::dispatchDidStartProvisionalLoad()
{
	m_view->SetEventState(WEBVIEW_EVENT_UNINIT);
}

void FrameLoaderClientWiiU::dispatchDidReceiveTitle(const StringWithDirection& title)
{
    CString cs = title.string().utf8();
	m_view->status(WEBVIEW_TITLE_CHANGED, cs.data());
}

void FrameLoaderClientWiiU::dispatchDidChangeIcons(WebCore::IconType)
{
    notImplemented();
}

void FrameLoaderClientWiiU::dispatchDidCommitLoad()
{
}

void FrameLoaderClientWiiU::dispatchDidFinishDocumentLoad()
{
	//This event tells the user that the document can be accessed and edited using the window handle returned by the load function
}

void FrameLoaderClientWiiU::dispatchDidFirstLayout()
{
    m_initLayoutCompleted = true;
}

void FrameLoaderClientWiiU::dispatchDidFirstVisuallyNonEmptyLayout()
{
}

void FrameLoaderClientWiiU::dispatchShow()
{
}

void FrameLoaderClientWiiU::cancelPolicyCheck()
{
    notImplemented();
}

void FrameLoaderClientWiiU::dispatchDidLoadMainResource(DocumentLoader*)
{
    notImplemented();
}

void FrameLoaderClientWiiU::revertToProvisionalState(DocumentLoader*)
{
    notImplemented();
}

void FrameLoaderClientWiiU::willChangeTitle(DocumentLoader*)
{
    // no need for, dispatchDidReceiveTitle is the right callback
}

void FrameLoaderClientWiiU::didChangeTitle(DocumentLoader *l)
{
    // no need for, dispatchDidReceiveTitle is the right callback
}

bool FrameLoaderClientWiiU::canHandleRequest(const ResourceRequest&) const
{
    notImplemented();
    return true;
}

bool FrameLoaderClientWiiU::canShowMIMETypeAsHTML(const String& MIMEType) const
{
    notImplemented();
    return false;
}

bool FrameLoaderClientWiiU::canShowMIMEType(const String& MIMEType) const
{
    //CRANK: DLM
    //We get an empty MIME type if the server returns a 304, indicating
    //that cached content is valid. If we've already cached the content,
    //it is displayable, so we can return true here...
    if (MIMEType.isEmpty())
        return true;

    if (MIMETypeRegistry::isSupportedImageMIMEType(MIMEType))
        return true;

    if (MIMETypeRegistry::isSupportedMediaMIMEType(MIMEType))
        return true;

    if (MIMETypeRegistry::isSupportedNonImageMIMEType(MIMEType))
        return true;

#if 0 // PluginDatabase is disabled until we have Plugin system done.
    if (PluginDatabase::installedPlugins()->isMIMETypeRegistered(MIMEType))
        return true;
#endif

    return false;
}

bool FrameLoaderClientWiiU::representationExistsForURLScheme(const String&) const
{
    return false;
}

String FrameLoaderClientWiiU::generatedMIMETypeForURLScheme(const String&) const
{
    notImplemented();
    return String();
}

void FrameLoaderClientWiiU::finishedLoading(DocumentLoader* loader)
{
    if (!m_pluginView) { 
        loader->writer()->setEncoding("", false);
    } else {
        m_pluginView->didFinishLoading();
        m_pluginView = 0;
        m_hasSentResponseToPlugin = false;
    }
}


void FrameLoaderClientWiiU::provisionalLoadStarted()
{
    notImplemented();
}

void FrameLoaderClientWiiU::didFinishLoad()
{
    notImplemented();
}

void FrameLoaderClientWiiU::prepareForDataSourceReplacement()
{
    notImplemented();
}

void FrameLoaderClientWiiU::setTitle(const StringWithDirection& title, const KURL& url)
{
    // no need for, dispatchDidReceiveTitle is the right callback
}

void FrameLoaderClientWiiU::dispatchDidReceiveContentLength(DocumentLoader*, unsigned long identifier, int lengthReceived)
{
    notImplemented();
}

void FrameLoaderClientWiiU::dispatchDidFinishLoading(DocumentLoader*, unsigned long identifier)
{
    //notImplemented();
	//m_view->SetEventState(WEBVIEW_EVENT_LOAD);
}

void FrameLoaderClientWiiU::dispatchDidFailLoading(DocumentLoader* loader, unsigned long identifier, const ResourceError& err)
{
    OS::Report("failed loading: ResourceError: %d %s %s\n",  err.errorCode(),
        err.failingURL().utf8().data(),
        err.localizedDescription().utf8().data());

	// Report error to system error handler...
	LibPlatform::Error::GetInstance()->Report(LibPlatform::Error::GetErrorCodeFromCurlError(err.errorCode()), L"FrameLoader cURL Error", (void*)err.errorCode());

    // Fixme: probably not an ideal solution but maybe this will make Kirstan not bug me for a while - NSA-LH 5/1/2012
    if (m_view) m_view->error(err.failingURL().utf8().data(), err.localizedDescription().utf8().data());

    notImplemented();
}

bool FrameLoaderClientWiiU::dispatchDidLoadResourceFromMemoryCache(DocumentLoader*, const ResourceRequest&, const ResourceResponse&, int length)
{
    notImplemented();
    return false;
}

void FrameLoaderClientWiiU::dispatchDidLoadResourceByXMLHttpRequest(unsigned long, const String&)
{
    notImplemented();
}

void FrameLoaderClientWiiU::dispatchDidFailProvisionalLoad(const ResourceError& err)
{
    dispatchDidFailLoad(err);
}

void FrameLoaderClientWiiU::dispatchDidFailLoad(const ResourceError& err)
{
    notImplemented();
}

void FrameLoaderClientWiiU::download(ResourceHandle*, const ResourceRequest& request, const ResourceRequest&, const ResourceResponse&)
{
    notImplemented();
}

// copied from WebKit/Misc/WebKitErrors[Private].h
enum {
    WebKitErrorCannotShowMIMEType = 100,
    WebKitErrorCannotShowURL = 101,
    WebKitErrorFrameLoadInterruptedByPolicyChange = 102,
    WebKitErrorCannotUseRestrictedPort = 103,
    WebKitErrorCannotFindPlugIn = 200,
    WebKitErrorCannotLoadPlugIn = 201,
    WebKitErrorJavaUnavailable = 202,
};

ResourceError FrameLoaderClientWiiU::cancelledError(const ResourceRequest& request)
{
    ResourceError error("Error", -999, request.url().string(),
                        "Request cancelled");
    error.setIsCancellation(true);
	m_view->error(request.url().string().utf8().data(), "Request Cancelled");
    return error;
}

ResourceError FrameLoaderClientWiiU::blockedError(const ResourceRequest& request)
{
	m_view->error(request.url().string().utf8().data(), "Request Blocked");
    return ResourceError("Error", WebKitErrorCannotUseRestrictedPort, request.url().string(),
                         "Request blocked");
}

ResourceError FrameLoaderClientWiiU::cannotShowURLError(const ResourceRequest& request)
{
	m_view->error(request.url().string().utf8().data(), "Cannot show URL");
    return ResourceError("Error", WebKitErrorCannotShowURL, request.url().string(),
                         "Cannot show URL");
}

ResourceError FrameLoaderClientWiiU::interruptedForPolicyChangeError(const ResourceRequest& request)
{
	m_view->error(request.url().string().utf8().data(), "Frame load interrupted by policy change");
    return ResourceError("Error", WebKitErrorFrameLoadInterruptedByPolicyChange,
                         request.url().string(), "Frame load interrupted by policy change");
}

ResourceError FrameLoaderClientWiiU::cannotShowMIMETypeError(const ResourceResponse& response)
{
    return ResourceError("Error", WebKitErrorCannotShowMIMEType, response.url().string(),
                         "Cannot show mimetype");
}

ResourceError FrameLoaderClientWiiU::fileDoesNotExistError(const ResourceResponse& response)
{
    return ResourceError("Error", -998 /* ### */, response.url().string(),
                         "File does not exist");
}

ResourceError FrameLoaderClientWiiU::pluginWillHandleLoadError(const ResourceResponse&)
{
    notImplemented();
    return ResourceError("Error", 0, "", "");
}

bool FrameLoaderClientWiiU::shouldFallBack(const ResourceError& error)
{
    return !(error.isCancellation() || (error.errorCode() == WebKitErrorFrameLoadInterruptedByPolicyChange));
}

bool FrameLoaderClientWiiU::canCachePage() const
{
    return false;
}

Frame* FrameLoaderClientWiiU::dispatchCreatePage(const NavigationAction&)
{
        return 0;
}

void FrameLoaderClientWiiU::dispatchDidBecomeFrameset(bool)
{
    notImplemented();
}

void FrameLoaderClientWiiU::dispatchUnableToImplementPolicy(const ResourceError&)
{
    notImplemented();
}

void FrameLoaderClientWiiU::setMainDocumentError(DocumentLoader* loader, const ResourceError& error)
{
    if (!m_pluginView)
        return;
    m_pluginView->didFail(error);
    m_pluginView = 0;
    m_hasSentResponseToPlugin = false;
}

void FrameLoaderClientWiiU::startDownload(const ResourceRequest&, const String& suggestedName)
{
    notImplemented();
}

void FrameLoaderClientWiiU::updateGlobalHistory()
{
    notImplemented();
}

void FrameLoaderClientWiiU::savePlatformDataToCachedFrame(CachedFrame*)
{
    notImplemented();
}

void FrameLoaderClientWiiU::transitionToCommittedFromCachedFrame(CachedFrame*)
{
    notImplemented();
}

void FrameLoaderClientWiiU::transitionToCommittedForNewPage()
{
    Page* page = m_frame->page();
    ASSERT(page);

    bool isMainFrame = m_frame == page->mainFrame();

    m_frame->setView(0);

    RefPtr<FrameView> frameView;
    if (isMainFrame) {
 	IntSize size(m_view->width(), m_view->height());
        frameView = FrameView::create(m_frame, size);
    } else
        frameView = FrameView::create(m_frame);

    frameView->setParentVisible(true);
    m_frame->setView(frameView);

    if (m_frame->ownerRenderer())
        m_frame->ownerRenderer()->setWidget(frameView);
}

PassRefPtr<FrameNetworkingContext> FrameLoaderClientWiiU::createNetworkingContext()
{
    return FrameNetworkingContextWiiU::create(m_frame);
}

}
