/*
 * Copyright (c) 2012 NINTENDO.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions, and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions, and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY NINTENDO AND ITS CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL NINTENDO OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <stddef.h>
#include <wtf/Platform.h>

#include <PlatformOS.h>
using namespace LibPlatform;

#include "Document.h"
#include "EventHandler.h"
#include "Frame.h"
#include "Node.h"
#include "Page.h"
#include "RenderView.h"
#include "RenderLayer.h"

#include "WiiUAutoScroller.h"
#include "WiiUScrollConstants.h"


const double WiiUAutoScroller::smoothScrollIncrement = 40.0;
const double WiiUAutoScroller::smoothScrollTimeIncrement = 0.016;



WiiUAutoScroller::WiiUAutoScroller()
	 : m_scrollNode(NULL),
	 m_XVelocity(0.0f),
	 m_YVelocity(0.0f),
	 m_lastTime(0.0f),
	 m_duration(0.0f),
	 m_acceleration(WiiUScrollConstants::default_acceleration()),
	 m_currentStepValue(0.0f),
	 m_startStepValue(0.0f),
	 m_endStepValue(0.0f),
	 m_animationDuration(0.0f),
	 m_animationInterval(0.0f),
	 m_elapsedTime(0.0f),
	 m_isScrolling(false),
	 m_velocityTimer(this, &WiiUAutoScroller::scrollVelocity),
	 m_smoothScrollTimer(this, &WiiUAutoScroller::scrollSmooth),
	 m_remainingX(0),
	 m_remainingY(0),
	 m_isScrollingForNav(false)
{
}

WiiUAutoScroller::~WiiUAutoScroller()
{
	stopScrollWithVelocity();
}

void WiiUAutoScroller::startScrollWithVelocity(float x_velocity, float y_velocity)
{
	if (m_acceleration >= 0.0f) {
		m_acceleration = WiiUScrollConstants::default_acceleration();
	}

	m_lastTime = 0.0f;
	m_XVelocity = x_velocity;
	m_YVelocity = y_velocity;

	m_duration = 120;

	//start the timer
	m_startStepValue = m_currentStepValue;
	m_endStepValue = 1.0f;

	if (m_duration == 0) {
		//skip to end of animation
	}else if (m_currentStepValue == m_endStepValue) {
		return;
	}

	m_animationDuration = m_duration * (1 - m_currentStepValue);
	m_animationInterval = calculateInterval(defaultFrameRateHz);
	m_velocityTimer.stop();

	m_isScrolling = true;
	m_isScrollingForNav = false;
	m_velocityTimer.startRepeating(m_animationInterval);
}

void WiiUAutoScroller::stopScrollWithVelocity()
{
	m_XVelocity = m_YVelocity = m_lastTime = m_duration = m_currentStepValue = m_elapsedTime = 0.0f;
	m_acceleration = WiiUScrollConstants::default_acceleration();
	m_startStepValue = 0.0f;
	m_endStepValue = 0.0f;
	m_animationDuration = 0.0f;
	m_animationInterval = 0.0f;
	//stop the timer
	if (isScrolling()) {
		m_velocityTimer.stop();
		m_isScrolling = false;
	}
}


void WiiUAutoScroller::startSmoothScroll(int x_diff, int y_diff)
{
	if(m_remainingX == 0)
		m_remainingX = x_diff;
	else
		m_remainingX += x_diff;
	if(m_remainingY == 0)
		m_remainingY = y_diff;
	else
		m_remainingY +=y_diff;

	if (!m_smoothScrollTimer.isActive()) {
		m_smoothScrollTimer.startOneShot(smoothScrollTimeIncrement);
	}
}

void WiiUAutoScroller::stopSmoothScroll()
{
	//stop the timer
	m_remainingX = 0;
	m_remainingY = 0;
	if (isScrolling()) {
		m_velocityTimer.stop();
		m_isScrolling = false;
	}
}

bool WiiUAutoScroller::shouldScrollWithVelocity(WebCore::Frame *mainFrame, int dx, int dy)
{
	if (!m_scrollNode) {
		return false;
	}

	WebCore::RenderObject* renderer = m_scrollNode->renderer();

	if (!renderer) {
		return false;
	}

	WebCore::RenderLayer* layer = renderer ? renderer->enclosingLayer() : 0;

	if (!layer) {
		return false;
	}

	bool isLineClampNoneByParent = true;

	if (renderer->parent()) {
		isLineClampNoneByParent = renderer->parent()->style()->lineClamp().isNone();
	}

	if (renderer->hasOverflowClip() && isLineClampNoneByParent) {
		int offsetX = layer->scrollXOffset() + dx;
		int offsetY = layer->scrollYOffset() + dy;

		if(offsetX < 0) {
			offsetX = 0;
		}

		if(offsetY < 0) {
			offsetY = 0;
		}

		if(!canScrollLayer(layer, offsetX, offsetY)) {
			setScrollNodeToMainDocument();
			return true;
		}

		return false;
	}

	WebCore::Frame* frame = renderer->frame();

	if (frame == mainFrame) {
		return true;
	}

	if(renderer->view()->frameView() && !canScrollView(renderer->view()->frameView(), dx, dy))
	{
		setScrollNodeToMainDocument();
		return true;
	}

	return false;
}

void WiiUAutoScroller::scrollEnded()
{
	stopScrollWithVelocity();
}

void WiiUAutoScroller::scrollProgressed()
{
	float t = m_currentStepValue * m_duration;
	float a_x = m_XVelocity > 0 ? m_acceleration : -m_acceleration;
	float a_y = m_YVelocity > 0 ? m_acceleration : -m_acceleration;
	float dx = getDelta(m_XVelocity, a_x, m_lastTime, t);
	float dy = getDelta(m_YVelocity, a_y, m_lastTime, t);

	m_XVelocity = m_XVelocity * (1 - m_currentStepValue);
	m_YVelocity = m_YVelocity * (1 - m_currentStepValue);
	m_lastTime = t;
	scrollNode(-1.0f * dx, -1.0f * dy);

	//if the node is no longer actually moving
	if(dx == 0.0f && dy == 0.0f)
	{
		//and the velocity time is active
		if(m_velocityTimer.isActive())
		{
			//Stop the scroll to prevent false positives from isScrolling()
			stopScrollWithVelocity();
		}
	}
}

void WiiUAutoScroller::scrollCanceled()
{
	stopScrollWithVelocity();
}

void WiiUAutoScroller::scrollVelocity(WebCore::Timer<WiiUAutoScroller>* timer)
{
	m_elapsedTime += m_animationInterval;
	m_currentStepValue = m_elapsedTime / m_animationDuration;
	if (m_currentStepValue >= 1.0) {
		m_currentStepValue = 1.0;
	}
	scrollProgressed();
	if (m_currentStepValue == 1.0) {
		stopScrollWithVelocity();
	}
}

void WiiUAutoScroller::scrollSmooth(WebCore::Timer<WiiUAutoScroller>* timer)
{
	int incrementX = 0;
	int incrementY = 0;
	double percentChange = 0.0;

	if(abs(m_remainingX) >= abs(m_remainingY)) {
		percentChange = fabs((double)m_remainingY)/fabs((double)m_remainingX);
		if(m_remainingY < 0) {
			incrementY = (smoothScrollIncrement * percentChange) * -1;
		} else {
			incrementY = smoothScrollIncrement * percentChange;
		}
		if(m_remainingX < 0)
			incrementX = smoothScrollIncrement * -1;
		else
			incrementX = smoothScrollIncrement;
	} else {
		percentChange = fabs((double)m_remainingX)/fabs((double)m_remainingY);
		if(m_remainingX < 0) {
			incrementX = (smoothScrollIncrement * percentChange) * -1;
		} else {
			incrementX = smoothScrollIncrement * percentChange;
		}
		if(m_remainingY < 0)
			incrementY = smoothScrollIncrement * -1;
		else
			incrementY = smoothScrollIncrement;
	}

	if(m_remainingX > 0) {
		if(m_remainingX < incrementX)
			incrementX = m_remainingX;
	} else if(m_remainingX < 0) {
		if(m_remainingX > incrementX)
			incrementX = m_remainingX;
	} else {
		incrementX = 0;
	}

	if(m_remainingY > 0) {
		if(m_remainingY < incrementY)
			incrementY = m_remainingY;
	} else if(m_remainingY < 0) {
		if(m_remainingY > incrementY)
			incrementY = m_remainingY;
	} else {
		incrementY = 0;
	}

	m_remainingX -= incrementX;
	m_remainingY -= incrementY;

	scrollNode(incrementX, incrementY);

	if((m_remainingX != 0 || m_remainingY != 0) && !m_smoothScrollTimer.isActive()) {
		m_smoothScrollTimer.startOneShot(smoothScrollTimeIncrement);
	} else if (m_remainingX == 0 && m_remainingY == 0) {
		m_isScrollingForNav = false;
	}

}


float WiiUAutoScroller::calculateInterval(int frameRate)
{
	float timerInterval = 1000000 / frameRate;
	if (timerInterval < 10000) {
		timerInterval = 10000;
	}
	return (timerInterval/1000000);
}

float WiiUAutoScroller::calculateTween(float state)
{
	//We're assuming animation to be ease out
	return 1.0f - pow(1.0f - state, 2);
}

float WiiUAutoScroller::getDelta(float v0, float a, float t1, float t2)
{
	return getPosition(v0, a, t2) - getPosition(v0, a, t1);
}

float WiiUAutoScroller::getPosition(float v0, float a, float t)
{
	float max_t = -v0/a;
	if (t > max_t) {
		t = max_t;
	}

	return t*(v0+0.5f*a*t);
}

void WiiUAutoScroller::scrollNode(int dx, int dy) {

	if(dx == 0 && dy == 0) {
		return;
	}

	if (!m_scrollNode) {
		return;
	}

	WebCore::RenderObject* renderer = m_scrollNode->renderer();
	WebCore::RenderLayer* layer = renderer ? renderer->enclosingLayer() : 0;

	if (!layer) {
		return;
	}

	bool isLineClampNoneByParent = true;
	if (renderer->parent()) {
		isLineClampNoneByParent = renderer->parent()->style()->lineClamp().isNone();
	}
	if (renderer->hasOverflowClip() && isLineClampNoneByParent) {
		WebCore::IntPoint scrollPoint(layer->scrollXOffset() + dx, layer->scrollYOffset() + dy);
		WebCore::IntPoint oldOffset(layer->scrollXOffset(), layer->scrollYOffset());


		WebCore::IntPoint newScrollPosition = scrollPoint.shrunkTo(layer->maximumScrollPosition());
		newScrollPosition = newScrollPosition.expandedTo(layer->minimumScrollPosition());

		if(oldOffset == newScrollPosition) {
			if(!m_velocityTimer.isActive())
				scrollMainFrame(dx,dy);
			else
				stopScrollWithVelocity();
			return;
		}

		layer->scrollToOffset(newScrollPosition.x(), newScrollPosition.y());

		WebCore::Frame* frame = renderer->frame();
		if(frame) {
			WebCore::EventHandler* eventHandler = frame->eventHandler();
			eventHandler->updateAutoscrollRenderer();
		}
	}else if (renderer->view()->frameView()) {
		WebCore::FrameView *fview = renderer->view()->frameView();
 		if(!canScrollView(fview, dx, dy)) {
 			if(!m_velocityTimer.isActive())
 				scrollMainFrame(dx,dy);
 			else
 				stopScrollWithVelocity();
 			return;
 		}
 		renderer->view()->frameView()->scrollBy(WebCore::IntSize(dx,dy));
	}
}


bool WiiUAutoScroller::canScrollLayer(WebCore::RenderLayer *layer, int scrollX, int scrollY)
{
	WebCore::IntPoint scrollMax = layer->maximumScrollPosition();
	WebCore::IntPoint scrollMin = layer->minimumScrollPosition();

	if(scrollX != 0 && scrollX <= scrollMax.x() && scrollX >= scrollMin.x()) {
		return true;
	}

	if(scrollY != 0 && scrollY <= scrollMax.y() && scrollY >= scrollMin.y()) {
		return true;
	}

	return false;
}

bool WiiUAutoScroller::canScrollView(WebCore::FrameView *view, int dx, int dy)
{
	WebCore::IntPoint offset = view->scrollPosition();
	WebCore::IntPoint scrollMax = view->maximumScrollPosition();
	WebCore::IntPoint scrollMin = view->minimumScrollPosition();

	offset.move(dx, dy);

	if(offset.x() != 0  && offset.x() <= scrollMax.x() && offset.x() >= scrollMin.x()) {
		return true;
	}

	if(offset.y() != 0 && offset.y() <= scrollMax.y() && offset.y() >= scrollMin.y()) {
		return true;
	}

	return false;
}

void WiiUAutoScroller::scrollMainFrame(int dx, int dy)
{
	WebCore::Document *dom = m_scrollNode->document();
	if(!dom) {
		return;
	}

	WebCore::Page *page = dom->page();
	if(!page) {
		return;
	}
	page->mainFrame()->view()->scrollBy(WebCore::IntSize(dx,dy));
}

void WiiUAutoScroller::setScrollNodeToMainDocument()
{
	WebCore::Document *dom = m_scrollNode->document();
	if(!dom) {
		m_scrollNode = NULL;
		return;
	}

	WebCore::Page *page = dom->page();
	if(!page) {
		m_scrollNode = NULL;
		return;
	}
	m_scrollNode = page->mainFrame()->document();
}


